/*
    KMines (not the KDE original version though ;-) - it's just a simple KDE app demo)
    Copyright (C) 2006 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "minefield.h"
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <qpopupmenu.h>

using namespace std;

Minefield::Minefield(QWidget *parent, int w, int h, int m): QGrid(w, parent), W(w), H(h), M(m)
{
	cells=new Cell**[H];
	int i, j, x, y;
 
	srand(time(0));
	for (i=0; i<H; i++) {
		cells[i]=new Cell*[W];
		for (j=0; j<W; j++) {
			cells[i][j]=new Cell(this, this, j, i);
		}
	}
	
	for (i=0; i<M; ) {
		x=(random()%W);
		y=(random()%H);
		if (cells[y][x]->state==0) {
			cells[y][x]->state=CELL_MINE;
			i++;
		}
	}
	playing=true;
}

Minefield::~Minefield()
{
	for (int i=0; i<H; i++) {
		for (int j=0; j<W; j++) delete cells[i][j];
		delete []cells[i];
	}
	delete []cells;
}

void Minefield::SetPlaying()
{
	playing=true;
}

bool Minefield::Playing()
{
	return playing;
}

void Minefield::ItemClicked(int x, int y)
{
	switch (explore(x, y)) {
	case 0: //ok
		if (isWon()) gameOver(1);
		break;
	case 1: //mine
		gameOver(0);
		break;
	}
}

void Minefield::ItemRightclicked(int x, int y)
{
	if ((cells[y][x]->state&(CELL_FLAG|CELL_UNSURE|CELL_SHOW)) == 0) {
		cells[y][x]->state|=CELL_FLAG;
		cells[y][x]->setText("*");
	} else {
		if ((cells[y][x]->state&CELL_FLAG) == CELL_FLAG) {
			cells[y][x]->state=(cells[y][x]->state& ~CELL_FLAG)|CELL_UNSURE;
			cells[y][x]->setText("?");
		} else if ((cells[y][x]->state&CELL_UNSURE) == CELL_UNSURE) {
			cells[y][x]->state=(cells[y][x]->state& ~CELL_UNSURE);
			cells[y][x]->setText(" ");
		}
	}
}

void Minefield::gameOver(int how)
{
	playing=false;
//show cells' real contents
	for (int i=0; i<H; i++) {
		for (int j=0; j<W; j++) {
			//cells[i][j]->setDown(true);
			if (cells[i][j]->state&CELL_MINE) {
				cells[i][j]->setText("@");
				if (!(cells[i][j]->state&CELL_FLAG)) cells[i][j]->setText("#");
			}
		}
	}
//finish it
	if (how==1) QMessageBox::information(this, "Game over", "Congrats, you won! ;-)", "OK");
	else QMessageBox::information(this, "Game over", "You are a looser, better try it again :-P", "OK");
}

Cell::Cell(Minefield *_mf, QWidget *_parent, int _x, int _y):QPushButton(_parent, "")
{
	mf=_mf;
	state=0;
	x=_x;
	y=_y;
	setText(" ");
	setFont(QFont("Helvetica", 10));
	setToggleButton(true);
	setFixedWidth(30);
	setFixedHeight(30);
	connect(this, SIGNAL(clicked()), this, SLOT(clicked()));
}

void Cell::clicked()
{
	mf->ItemClicked(x, y);
	setDown(true);
}

void Cell::mouseReleaseEvent(QMouseEvent *e)
{
	if (!mf->Playing()) {
		setDown(false);
		return;
	}

	if (e->button() & RightButton) {
		mf->ItemRightclicked(x, y);
	} else if (e->button() & LeftButton) {
		emit clicked();
	}
}
